/************************************************************************/
/*  uepoch - convert time to unix epoch and vice versa			*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: uepoch.c,v 1.1 2000/10/26 13:40:30 doug Exp $ ";
#endif

/************************************************************************
Modification History
------------------------------------------------------------------------
Date	    Ver			Who	What
------------------------------------------------------------------------
2000/10/26  1.1 (2000.300)	DSN	Add copyright.
2000/04/24  1.0	(2000.115)	DSN	Initial coding.
/************************************************************************/

#include    <stdio.h>
#define	    VERSION	"1.1 (2000.300)"
   
char *syntax[] = {
"%s version " VERSION " -- convert between date/time and unix epoch time.",
"%s  -n | -t [-h] epoch_time | date_time",
"    where:",
"	-n	    Specify nominal epoch time (no leap seconds).",
"	-t	    Specify true epoch time (include leap seconds).",
"	-q	    quiet - output just the converted time.",
"	-h	    Help - prints syntax message.",
"	epoch_time | date_time",
"		    Input time.  If epoch_time, convert to date_time.",
"		    If date_time, convert to epoch_time.",
"		    Epoch time is relative to Jan 1, 1970.",
NULL };

/************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include <qlib2.h>

#define	info		stdout

/************************************************************************/
/*  External variables and symbols.					*/
/************************************************************************/
char *cmdname;			/* program name from command line.	*/
int date_fmt = JULIAN_FMT_1;
#define	PRINT_EPOCH	1
#define	PRINT_DATETIME	2

/************************************************************************/
/*  main:   main program.
/************************************************************************/
main (argc, argv)
    int		argc;
    char	**argv;
{
    INT_TIME it, *pt;
    double epoch;
    int nominal = -1;
    char *typestr;
    int quiet = 0;
    int output;

    /* Variables needed for getopt. */
    extern char	*optarg;
    extern int	optind, opterr;
    int		c;
    char	*p;

    cmdname = ((p = strrchr(*argv,'/')) != NULL) ? ++p : *argv;
    /*	Parse command line options.					*/
    while ( (c = getopt(argc,argv,"hntqD:")) != -1) switch (c) {
	case '?':
	case 'h':   print_syntax(cmdname,syntax,info); exit(0); break;
	case 'n':   nominal = 1; break;
	case 't':   nominal = 0; break;
	case 'q':   quiet = 1; break;
	case 'D':
	    /* Date display format.   */
	    if (strcmp (optarg, "j,") == 0) date_fmt = JULIAN_FMT_1;
	    else if (strcmp (optarg, "m") == 0) date_fmt = MONTHS_FMT_1;
	    else {
		fprintf (info, "Error: invalid format for date display: %s\n", optarg);
		exit(1);
	    }
	    break;
	default:    fprintf (info, "Unknown option: -%c\n", c); exit(1);
    }
    /*	Skip over all options and their arguments.			*/
    argv = &(argv[optind]);
    argc -= optind;

    if (argc <= 0) { 
	fprintf (info, "Error: missing epoch time or date.\n");
	exit(1);
    }
    if (nominal < 0) {
	fprintf (info, "Error: missing -n or -t flag.\n");
	exit(1);
    }
	
    epoch = strtod(argv[0], &p);
    if (*p == '\0') {
	struct timeval tv;
	output = PRINT_DATETIME;
	epoch = atof(argv[0]);
	if (fabs(epoch) > IHUGE) {
	    fprintf (stderr, "Error: epoch time exceeds bounds\n");
	    exit(1);
	}
	if (nominal) {
	    tv.tv_sec = (int)epoch;
	    tv.tv_usec = (epoch - tv.tv_sec) * USECS_PER_SEC;
	    it = int_time_from_timeval (&tv);
	    typestr = "Nominal";
	}
	else {
	    it = tepoch_to_int (epoch);
	    typestr = "True";
	}
    }
    else {
	output = PRINT_EPOCH;
	if ((pt = parse_date(argv[0]))) {
	    it = *pt;
	}
	else {
	    fprintf (info, "Error: Unrecognizable input: %s\n", argv[0]);
	    exit(1);
	}
	if (nominal) {
	    time_t clock;
	    clock = unix_time_from_int_time(it);
	    epoch = clock + (double)(it.usec/USECS_PER_SEC);
	    typestr = "Nominal";
	}
	else {
	    epoch = int_to_tepoch (it);
	    typestr = "True";
	}
    }
    if (quiet) {
	if (output == PRINT_DATETIME) {
	    printf ("%s\n", time_to_str(it, date_fmt));
	}
	else {
	    printf ("%.4lf", epoch);
	}
    }
    else {
	printf ("%s epoch time %.4lf = %s\n", typestr, epoch, time_to_str(it, date_fmt));
    }
    return (0);
}
